/**
 * 全局配置
 */
interface GlobalCfg {
    /**
     * 远程的配置路径
     */
    remote?: string;
    
    /**
     * 项目名称
     */
    project: string;

    /**
     * 客户端配置导出路径
     */
    clientPath: string;

    /**
     * 服务端配置导出路径
     */
    serverPath?: string;

    /**
     * 服务端用于注册常量和引用的类
     */
    serverRegClass?: [string, string];

    /**
     * 客户端用于注册常量和引用的类
     */
    clientRegClass?: [string, string];
    /**
     * 执行完成后调用的脚本
     */
    endScript?: string;

    /**
     * 执行完成后调用的url
     */
    endAction?: string;

    /**
     * 屏蔽字源文件名称
     */
    dirtyName?: string;

    /**
     * 保存服务端屏蔽字数据文件名称
     */
    dirtyServerName?: string;

    /**
     * 保存客户端屏蔽字数据文件名称
     */
    dirtyClientName?: string;

    /**
     * 客户端屏蔽字文件保存路径
     */
    dirtyClientPath?: string;
    

    /**
     * 名字库源文件名称
     */
    nameLib?: string;

    /**
     * 保存客户端数据文件名称
     */
    nameLibClientName?: string;

    /**
     * 名字库源文件保存路径
     */
    nameLibClientPath?: string;

    /**
     * 是否自动更新svn
     */
    isAutoSvn?: boolean;
}

/**
 * 属性定义
 */
interface ProDefine {

    /**
     * 属性名称
     */
    name: string;

    /**
     * 描述
     */
    desc: string;

    /**
     * 是否导出客户端数据
     * 0 不解析
     * 1 解析，并在代码生成时，生成类型对应的字段
     * 2 解析，生成代码时，在decode方法中，由临时变量记录数据，不生成方法
     */
    client: number;

    /**
     * 是否导出服务端数据
     * 0 不解析
     * 1 解析，并在代码生成时，生成类型对应的字段
     * 2 解析，生成代码时，在decode方法中，由临时变量记录数据，不生成方法
     */
    server: number;

    /**
     * 数据类型检查器
     */
    checker: TypeChecker;

    /**
     * 数据类型服务器解析用
     */
    type: String;
}

/**
 * 检查数据是否符合类型，否则抛错
 * 
 * @interface TypeChecker
 */
interface TypeChecker {

    type: string;

    /**
     * java类型
     */
    javaType:string;

    /**
     * 类型索引值
     */
    idx: number;

    /**
     * 检查并返回处理后的数据
     * 
     * @param {string} value 待处理的数据
     * @returns {any} 
     */
    check(value: string): any;

    /**
     * 检查并返回处理后的服务器数据
     */
    checkServer(value: string): any;

    solveString?: string;

    solveJavaString?:string;
}

interface IPluginData {
    gcfg: GlobalCfg;
    /**
     * 文件名
     * @type {string}
     */
    filename: string;
    /**原始数据 */
    rawData: any[];
    /**服务器数据 */
    sdatas: any[];
    /**客户端数据 */
    cdatas: any[];
    /**
     * 列定义
     */
    defines: ProDefine[];
    /**
     * 数据起始行
     */
    dataRowStart: number;

    /**
     * 行配置对应的行数
     * Key  {string}    配置名称     
            "程序配置内容": "cfgRow",
            "前端解析": "clientRow",
            "后端解析": "serverRow",
            "默认值": "defaultRow",
            "数据类型": "typeRow",
            "描述": "desRow",
            "属性名称": "nameRow"

       Value {number}   行号
     * 
     * @type {{ [index: string]: number }}
     */
    rowCfg: { [index: string]: number };
}

/**
 * 
 * 插件回调
 * @interface IPluginCallback
 */
interface IPluginCallback {

    /**
     * 
     * 
     * @param {Error} [err]           错误信息
     * @param {string} [output]     输出的字符串
     * @param {any[]} [sdatas]      处理后的服务端数据
     * @param {any[]} [cdatas]      处理后的客户端数据
     */
    (err: Error, output?: string, sdatas?: any[], cdatas?: any[]): void
}

/**
 * 
 * 插件定义
 * @interface IPlugin
 */
interface IPlugin {

    /**
     * 
     * 运行插件
     * @param {IPluginData} data
     * @param {IPluginCallback} callback
     */
    execute(data: IPluginData, callback: IPluginCallback);
}

interface IPluginLoaderCallback {
    type: string;
    error?: number;
    err?: Error;
    output?: string;
    sdatas?: any[];
    cdatas?: any[];
}