var fs = nodeRequire("fs");
/**
 * 手写代码的默认提示
 */
const ManualCodeDefaultComment = {
	/**
	 * 类上方提示
	 */
    $area1: "!这里填写类上方的手写内容!",
	/**
	 * 类中提示
	 */
    $area2: "!这里填写类里面的手写内容!",
	/**
	 * 类下方提示
	 */
    $area3: "!这里填写类下发的手写内容!",
	/**
	 * 处理函数提示
	 */
    $decode: "!这里填写方法中的手写内容!",
}

/**
 * 生成手动代码区域的文本
 */
function genManualAreaCode(key: string, cinfo: { [index: string]: string }) {
    let yuanyou = cinfo[key];
    let tab = `\t`;
    if (key == "$area2") tab += `\t`;
    if (key == "$decode") tab += `\t\t`;
    let desc = `${ManualCodeDefaultComment[key]}`;
    return `/*-*begin ${key} ${desc}*-*/
${tab}${yuanyou?yuanyou:""}
${tab}/*-*end ${key}*-*/`
}

/**
 * 获取手动写的代码信息
 */
function getManualCodeInfo(file: string) {
    let dict: { [index: string]: string } = {};
    if (fs.existsSync(file)) {
        //读取文件内容
        let content = fs.readFileSync(file, "utf8");
        //找到手写内容
        let reg = /[/][*]-[*]begin[ ]([$]?[a-zA-Z0-9]+)[ ](.*?)[*]-[*][/]([^]*?)[/][*]-[*]end[ ]\1[*]-[*][/]/g;
        while (true) {
            let result = reg.exec(content);
            if (result) {
                let key = result[1];
                let manual = result[3].trim();
                if (!manual) {//没有注释
                    continue;
                } else if (key in ManualCodeDefaultComment) {
                    if (ManualCodeDefaultComment[key] == manual) {//类上中下的注释
                        continue;
                    }
                }
                dict[key] = manual;
            } else {
                break;
            }
        }
    }
    return dict;
}

function getGConfigCodeInfo(file: string, scriptName: string, sortName?: string) {
    sortName = sortName ? `, "${sortName}"` : "";
    let addstr = `rP("${scriptName}", ${scriptName}Cfg${sortName});`;
    if (fs.existsSync(file)) {
        //读取文件内容
        let content: string = fs.readFileSync(file, "utf8");
        if (content.indexOf(`${addstr}`) != -1) return "";
        let sstr = "//begin";
        let estr = "//end";
        let si = content.indexOf(sstr);
        let ei = content.indexOf(estr);
        if (ei > si && si != -1){
            let fcon = content.substring(si + sstr.length, ei).replace(/\t/g, "").replace(/\r/g, "");
            let temp = fcon.split("\n");//`rP("${tempfilename}", chuanqi.${tempfilename}Cfg);`
            let rarg: string[] = [];
            for (let i = 0, l = temp.length; i < l; i++) {
                let s = temp[i];
                if (s && s.indexOf(`rP("${scriptName}`) == -1){
                    rarg.push(temp[i]);
                }
            }
            rarg.push(addstr);
            console.log(rarg);
            return rarg.join("\n\t\t");
        }
        return addstr;
    }
    else {
        return addstr;
    }
}

export {genManualAreaCode, getManualCodeInfo, ManualCodeDefaultComment, getGConfigCodeInfo}